# Generated by roxygen2: do not edit by hand

S3method(print,camtrapdp)
export("contributors<-")
export("deployments<-")
export("media<-")
export("observations<-")
export(check_camtrapdp)
export(contributors)
export(deployments)
export(events)
export(example_dataset)
export(filter_deployments)
export(filter_media)
export(filter_observations)
export(individuals)
export(locations)
export(media)
export(merge_camtrapdp)
export(observations)
export(read_camtrapdp)
export(round_coordinates)
export(shift_time)
export(taxa)
export(update_taxon)
export(version)
export(write_camtrapdp)
export(write_dwc)
export(write_eml)
import(rlang)
importFrom(dplyr,.data)
importFrom(memoise,memoise)
